#!/usr/bin/perl -w
# Setting up prefs, directories, etc
use Cwd;
use strict;
use Apache::Htpasswd;

#my $nobody_id = getpwnam("nobody");
#my $user_id = getpwnam("www-data");
#my $group_id = getpwnam("www-data");

my $sep_line = "------------------------\n";


print "--------[SET-UP]--------\n";

#----------------------------------------# get correct id



my ($id, $user_id, $group_id);
if ( getpwnam("www-data") ) {
	# set id/group as www-data
	$id = getpwnam("www-data");
	($user_id, $group_id) = ($id, $id);
	print "User/group set to www-data/www-data [$user_id:$group_id]\n";
} elsif ( getpwnam("nobody") ) {
	# set id/group as nobody
	$id = getpwnam("nobody");
	($user_id, $group_id) = ($id,$id);
	print "User/group set to nobody/nobody [$user_id:$group_id]\n";
}

print "Do you wish to enter a user/group different from the above? [y/n]: ";
my $own_answer = <STDIN>; chomp $own_answer;
$own_answer = lc($own_answer);
my ($a_user, $a_grp);
OUTER: while(! $a_user && ! $a_grp) {
	if($own_answer eq "y") {
		print "User (i.e. \"www-user\"): ";
		$a_user = <STDIN>; chomp $a_user;
		print "Group (i.e. \"nogroup\"): ";
		$a_grp = <STDIN>; chomp $a_grp;
		if ( getpwnam("$a_user") && getpwnam("$a_grp") ) {
			$user_id = getpwnam("$a_user");
			$group_id = getpwnam("$a_grp");
			print "User/group both found and set to $a_user:$a_grp\n";
		} else {
			print "System cannot identify either or both user/group entered, please try again.\n";
			redo OUTER;
		}
	} elsif($own_answer eq "n") {
		$a_user = "$user_id";
		$a_grp = "$group_id";
	} else {
		# not an answer
		print "\nUnrecognized option. Try again...\n";
		next;
	}
}
#----------------------------------------#  Set up directories:
my @dirs = ("admin", "users", "templates");
foreach my $d (@dirs) {
	if (-e $d) {
		print "Please remove \"$d\" manually (not overwritten)\n";
	} else {
		mkdir("$d", 0777);
		chown($user_id, $group_id, "$d");
		print "Created new \"$d\" directory\n";
	}
}
#----------------------------------------#  create the password file:
# create the "admin" account.

print $sep_line;

if (-e "admin/passwords") {
	print "Not over-writing passwords file, please remove manually\n";
} else {
	open(PWD, ">admin/passwords");

	my $passf = new Apache::Htpasswd("admin/passwords");

	$passf->htpasswd("admin", "admin");

	print 	"Created password file, and created an \"admin\" account.\n",
		"Password: \"admin\" (please change with exp_admin)\n";

	chmod(0775, "admin/passwords");
	# Change the ownership of passwords to nobody:
	# MIGHT HAVE TO ALTER THIS TO SUIT ALL SET-UPS
	chown($user_id, $group_id, "admin/passwords");

	close(PWD);
}

#----------------------------------------#  create sample options

print $sep_line;

my %def_opt;

if (-e "templates/options.db") {
	print "Not over-writing options.db file, please remove manually.\n";
} else {
	dbmopen(%def_opt, "templates/options", 0777) || die "Cannot create options: $!";

	$def_opt{"cat_menu"} = "bills!!entertainment!!groceries!!medical!!transport!!snacks";

	print "Created a default category menu in templates directory.\n";

	dbmclose(%def_opt);

	# This work with 5.00x, maybe not 5.6.* # put all three, just in case
	chown($user_id, $group_id, "templates/options.db");
	chown($user_id, $group_id, "templates/options.dir");
	chown($user_id, $group_id, "templates/options.pag");
}
#----------------------------------------#  Edit 'exp' and 'exp_admin':
# change the default directories.

print $sep_line;


### Create the full path.
my $pwd = cwd;
my ($full_path, $rel_path);
until ($full_path) {
	print "Is this your Full path to \"exp\" and \"exp_admin\" 
[$pwd] [y|n]: ";
	my $answer = <STDIN>; chomp $answer;
	$answer = lc($answer);

	if ($answer eq "y") {
		$full_path = "$pwd";
	} elsif ($answer eq "n") {
		print "\nPlease enter the full path to the \"exp\" and \"exp_admin\" scripts 
[i.e. \"\/usr\/lib\/cgi-bin\/sem\"] (no trailing slash!): ";
		$full_path = <STDIN>; chomp $full_path;
	} else {
		print "\nUnrecognized option. Try again...\n";
		next;
	}
} # end until

print $sep_line;

#### Create the relative path:
until ($rel_path) {
	my @pwd_split = split(/\//, $pwd); my $last_child = $pwd_split[-1];
	print "NEXT ENTER HTTP SERVER PATH: \"i.e. http:\/\/yourserver\/cgi-bin\/$last_child\" ";
	print "\nIs this your http server path to \"exp\" and \"exp_admin\" 
[/cgi-bin/$last_child] ? [y|n]: ";
	my $answer = <STDIN>; chomp $answer;
	$answer = lc($answer);

	if ($answer eq "y") {
		$rel_path = "/cgi-bin/$last_child";
	} elsif ($answer eq "n") {
		print 	"\nPlease enter the server path to \"exp\" 
and \"exp_admin\" scripts [i.e. \"\/cgi-bin\/$last_child\"]
(no trailing slash!): ";
		$rel_path = <STDIN>; chomp $rel_path;
	} else {
		print "\nUnrecognized option. Try again...\n";
		next;
	}
} # end until


## read "exp"
open(EXP, "exp");
open(EXP_ADMIN, "exp_admin");
my @f = <EXP>;
my @f_a = <EXP_ADMIN>;
close(EXP_ADMIN);
close(EXP);

my $x;

# change EXP
my $f_sz = @f;
for($x = 0; $x < $f_sz; $x++) {
	if ( $f[$x] =~ m/my \$def_cgi_full/ ){
		$f[$x] = "my \$def_cgi_full = \"$full_path\";\n";
	}

	if ( $f[$x] =~ m/my \$def_cgi_root/ ){
		$f[$x] = "my \$def_cgi_root = \"$rel_path\";\n";
	}
	
}


# NOW CHANGE EXP_ADMIN
my $f_a_sz = @f_a;
for($x = 0; $x < $f_a_sz; $x++) {
	if ( $f_a[$x] =~ m/my \$def_cgi_full/ ){
		$f_a[$x] = "my \$def_cgi_full = \"$full_path\";\n";
	}

	if ( $f_a[$x] =~ m/my \$def_cgi_root/ ){
		$f_a[$x] = "my \$def_cgi_root = \"$rel_path\";\n";
	}
	
}

open(NEW_EXP, ">exp");
	print NEW_EXP @f;
close(NEW_EXP);

open(NEW_EXP_ADMIN, ">exp_admin");
	print NEW_EXP_ADMIN @f_a;
close(NEW_EXP_ADMIN);

print $sep_line;

print 	"\nFULL PATH: $full_path\n",
	"SERVER PATH: $rel_path (http://yourserver$rel_path)\n",
	"\nOpen http://yourserver$rel_path/exp_admin to configure SEM\n",
	"\nAll done. Run the script again to alter default paths.\n",
	"\n*** YOU CAN RE-RUN THIS SCRIPT AND NOT DESTROY YOUR OLD ",
	"FILES AND DIRECTORIES. YOU WILL NEED TO MANUALLY ERASE ",
	"THEM IF YOU WANT TO START AGAIN.\n";
